package com.sun.org.apache.xalan.internal.lib;

import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import com.sun.org.apache.xpath.internal.objects.XObject;
import org.w3c.dom.Document;
import com.sun.org.apache.xpath.internal.objects.XBoolean;
import com.sun.org.apache.xpath.internal.objects.XNumber;
import jdk.xml.internal.JdkXmlUtils;
import com.sun.org.apache.xpath.internal.objects.XNodeSet;
import com.sun.org.apache.xpath.internal.NodeSet;
import javax.xml.transform.TransformerException;
import com.sun.org.apache.xpath.internal.XPath;
import com.sun.org.apache.xpath.internal.NodeSetDTM;
import org.xml.sax.SAXNotSupportedException;
import com.sun.org.apache.xalan.internal.res.XSLMessages;
import com.sun.org.apache.xpath.internal.XPathContext;
import org.w3c.dom.NodeList;
import com.sun.org.apache.xalan.internal.extensions.ExpressionContext;

public class ExsltDynamic extends ExsltBase
{
    public static final String EXSL_URI = "http://exslt.org/common";
    
    public ExsltDynamic() {
    }
    
    public static double max(ExpressionContext myContext, NodeList nl, String expr) throws SAXNotSupportedException {
        XPathContext xctxt = null;
        if ((myContext instanceof XPathContext.XPathExpressionContext)) {
            xctxt = ((XPathContext.XPathExpressionContext)myContext).getXPathContext();
            if (expr == null || expr.length() == 0) {
                return Double.NaN;
            }
            NodeSetDTM contextNodes = new NodeSetDTM(nl, xctxt);
            xctxt.pushContextNodeList(contextNodes);
            double maxValue = -1.7976931348623157E308;
            for (int i = 0; i < contextNodes.getLength(); ++i) {
                int contextNode = contextNodes.item(i);
                xctxt.pushCurrentNode(contextNode);
                double result = 0.0;
                try {
                    XPath dynamicXPath = new XPath(expr, xctxt.getSAXLocator(), xctxt.getNamespaceContext(), 0);
                    result = dynamicXPath.execute(xctxt, contextNode, xctxt.getNamespaceContext()).num();
                }
                catch (TransformerException e) {
                    xctxt.popCurrentNode();
                    xctxt.popContextNodeList();
                    return Double.NaN;
                }
                xctxt.popCurrentNode();
                if (result > maxValue) {
                    maxValue = result;
                }
            }
            xctxt.popContextNodeList();
            return maxValue;
        }
        else {
            throw new SAXNotSupportedException(XSLMessages.createMessage("ER_INVALID_CONTEXT_PASSED", new Object[] { myContext }));
        }
    }
    
    public static double min(ExpressionContext myContext, NodeList nl, String expr) throws SAXNotSupportedException {
        XPathContext xctxt = null;
        if ((myContext instanceof XPathContext.XPathExpressionContext)) {
            xctxt = ((XPathContext.XPathExpressionContext)myContext).getXPathContext();
            if (expr == null || expr.length() == 0) {
                return Double.NaN;
            }
            NodeSetDTM contextNodes = new NodeSetDTM(nl, xctxt);
            xctxt.pushContextNodeList(contextNodes);
            double minValue = Double.MAX_VALUE;
            for (int i = 0; i < nl.getLength(); ++i) {
                int contextNode = contextNodes.item(i);
                xctxt.pushCurrentNode(contextNode);
                double result = 0.0;
                try {
                    XPath dynamicXPath = new XPath(expr, xctxt.getSAXLocator(), xctxt.getNamespaceContext(), 0);
                    result = dynamicXPath.execute(xctxt, contextNode, xctxt.getNamespaceContext()).num();
                }
                catch (TransformerException e) {
                    xctxt.popCurrentNode();
                    xctxt.popContextNodeList();
                    return Double.NaN;
                }
                xctxt.popCurrentNode();
                if (result < minValue) {
                    minValue = result;
                }
            }
            xctxt.popContextNodeList();
            return minValue;
        }
        else {
            throw new SAXNotSupportedException(XSLMessages.createMessage("ER_INVALID_CONTEXT_PASSED", new Object[] { myContext }));
        }
    }
    
    public static double sum(ExpressionContext myContext, NodeList nl, String expr) throws SAXNotSupportedException {
        XPathContext xctxt = null;
        if ((myContext instanceof XPathContext.XPathExpressionContext)) {
            xctxt = ((XPathContext.XPathExpressionContext)myContext).getXPathContext();
            if (expr == null || expr.length() == 0) {
                return Double.NaN;
            }
            NodeSetDTM contextNodes = new NodeSetDTM(nl, xctxt);
            xctxt.pushContextNodeList(contextNodes);
            double sum = 0.0;
            double result;
            for (int i = 0; i < nl.getLength(); sum += result, ++i) {
                int contextNode = contextNodes.item(i);
                xctxt.pushCurrentNode(contextNode);
                result = 0.0;
                try {
                    XPath dynamicXPath = new XPath(expr, xctxt.getSAXLocator(), xctxt.getNamespaceContext(), 0);
                    result = dynamicXPath.execute(xctxt, contextNode, xctxt.getNamespaceContext()).num();
                }
                catch (TransformerException e) {
                    xctxt.popCurrentNode();
                    xctxt.popContextNodeList();
                    return Double.NaN;
                }
                xctxt.popCurrentNode();
            }
            xctxt.popContextNodeList();
            return sum;
        }
        else {
            throw new SAXNotSupportedException(XSLMessages.createMessage("ER_INVALID_CONTEXT_PASSED", new Object[] { myContext }));
        }
    }
    
    public static NodeList map(ExpressionContext myContext, NodeList nl, String expr) throws SAXNotSupportedException {
        XPathContext xctxt = null;
        Document lDoc = null;
        if ((myContext instanceof XPathContext.XPathExpressionContext)) {
            xctxt = ((XPathContext.XPathExpressionContext)myContext).getXPathContext();
            if (expr == null || expr.length() == 0) {
                return new NodeSet();
            }
            NodeSetDTM contextNodes = new NodeSetDTM(nl, xctxt);
            xctxt.pushContextNodeList(contextNodes);
            NodeSet resultSet = new NodeSet();
            resultSet.setShouldCacheNodes(true);
            for (int i = 0; i < nl.getLength(); ++i) {
                int contextNode = contextNodes.item(i);
                xctxt.pushCurrentNode(contextNode);
                XObject object = null;
                try {
                    XPath dynamicXPath = new XPath(expr, xctxt.getSAXLocator(), xctxt.getNamespaceContext(), 0);
                    object = dynamicXPath.execute(xctxt, contextNode, xctxt.getNamespaceContext());
                    if ((object instanceof XNodeSet)) {
                        NodeList nodelist = null;
                        nodelist = ((XNodeSet)object).nodelist();
                        for (int k = 0; k < nodelist.getLength(); ++k) {
                            Node n = nodelist.item(k);
                            if (!resultSet.contains(n)) {
                                resultSet.addNode(n);
                            }
                        }
                    }
                    else {
                        if (lDoc == null) {
                            lDoc = JdkXmlUtils.getDOMDocument();
                        }
                        Element element = null;
                        if ((object instanceof XNumber)) {
                            element = lDoc.createElementNS("http://exslt.org/common", "exsl:number");
                        }
                        else if ((object instanceof XBoolean)) {
                            element = lDoc.createElementNS("http://exslt.org/common", "exsl:boolean");
                        }
                        else {
                            element = lDoc.createElementNS("http://exslt.org/common", "exsl:string");
                        }
                        Text textNode = lDoc.createTextNode(object.str());
                        element.appendChild(textNode);
                        resultSet.addNode(element);
                    }
                }
                catch (Exception e) {
                    xctxt.popCurrentNode();
                    xctxt.popContextNodeList();
                    return new NodeSet();
                }
                xctxt.popCurrentNode();
            }
            xctxt.popContextNodeList();
            return resultSet;
        }
        else {
            throw new SAXNotSupportedException(XSLMessages.createMessage("ER_INVALID_CONTEXT_PASSED", new Object[] { myContext }));
        }
    }
    
    public static XObject evaluate(ExpressionContext myContext, String xpathExpr) throws SAXNotSupportedException {
        if ((myContext instanceof XPathContext.XPathExpressionContext)) {
            XPathContext xctxt = null;
            try {
                xctxt = ((XPathContext.XPathExpressionContext)myContext).getXPathContext();
                XPath dynamicXPath = new XPath(xpathExpr, xctxt.getSAXLocator(), xctxt.getNamespaceContext(), 0);
                return dynamicXPath.execute(xctxt, myContext.getContextNode(), xctxt.getNamespaceContext());
            }
            catch (TransformerException e) {
                return new XNodeSet(xctxt.getDTMManager());
            }
        }
        else {
            throw new SAXNotSupportedException(XSLMessages.createMessage("ER_INVALID_CONTEXT_PASSED", new Object[] { myContext }));
        }
    }
    
    public static NodeList closure(ExpressionContext myContext, NodeList nl, String expr) throws SAXNotSupportedException {
        XPathContext xctxt = null;
        if ((myContext instanceof XPathContext.XPathExpressionContext)) {
            xctxt = ((XPathContext.XPathExpressionContext)myContext).getXPathContext();
            if (expr == null || expr.length() == 0) {
                return new NodeSet();
            }
            NodeSet closureSet = new NodeSet();
            closureSet.setShouldCacheNodes(true);
            NodeList iterationList = nl;
            do {
                NodeSet iterationSet = new NodeSet();
                NodeSetDTM contextNodes = new NodeSetDTM(iterationList, xctxt);
                xctxt.pushContextNodeList(contextNodes);
                for (int i = 0; i < iterationList.getLength(); ++i) {
                    int contextNode = contextNodes.item(i);
                    xctxt.pushCurrentNode(contextNode);
                    XObject object = null;
                    try {
                        XPath dynamicXPath = new XPath(expr, xctxt.getSAXLocator(), xctxt.getNamespaceContext(), 0);
                        object = dynamicXPath.execute(xctxt, contextNode, xctxt.getNamespaceContext());
                        if ((object instanceof XNodeSet)) {
                            NodeList nodelist = null;
                            nodelist = ((XNodeSet)object).nodelist();
                            for (int k = 0; k < nodelist.getLength(); ++k) {
                                Node n = nodelist.item(k);
                                if (!iterationSet.contains(n)) {
                                    iterationSet.addNode(n);
                                }
                            }
                        }
                        else {
                            xctxt.popCurrentNode();
                            xctxt.popContextNodeList();
                            return new NodeSet();
                        }
                    }
                    catch (TransformerException e) {
                        xctxt.popCurrentNode();
                        xctxt.popContextNodeList();
                        return new NodeSet();
                    }
                    xctxt.popCurrentNode();
                }
                xctxt.popContextNodeList();
                iterationList = iterationSet;
                for (int i = 0; i < iterationList.getLength(); ++i) {
                    Node n2 = iterationList.item(i);
                    if (!closureSet.contains(n2)) {
                        closureSet.addNode(n2);
                    }
                }
            } while (iterationList.getLength() > 0);
            return closureSet;
        }
        else {
            throw new SAXNotSupportedException(XSLMessages.createMessage("ER_INVALID_CONTEXT_PASSED", new Object[] { myContext }));
        }
    }
}
